#  Copyright (c) 1997-2011
#  Ewgenij Gawrilow, Michael Joswig (Technische Universitaet Darmstadt, Germany)
#  http://www.polymake.de
#
#  This program is free software; you can redistribute it and/or modify it
#  under the terms of the GNU General Public License as published by the
#  Free Software Foundation; either version 2, or (at your option) any
#  later version: http://www.gnu.org/licenses/gpl.txt.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#-------------------------------------------------------------------------------
#  $Project: polymake $$Id: app.make 10614 2012-02-28 15:51:33Z benmuell $

#  Building an application, possibly in an extension or private wrapper depot.

AppName ?= $(notdir ${CURDIR})
AppTop ?= ${ProjectTop}
SourceTop := $(firstword ${ExtensionTop} ${AppTop})
SourceDir := ${SourceTop}/apps/${AppName}/src

-include ${SourceDir}/Makefile.inc

ifeq (${SharedModules},ALL)
  SharedModules :=
endif
ifeq (${SharedModules},)
  _broken := $(notdir $(call _get_no_export,${SourceDir},broken))
  SharedModules := $(basename $(filter-out ${_broken},$(call _list_CXX_sources,${SourceDir})))
else
  SharedModules := $(basename $(notdir $(wildcard $(foreach s,${CXX_suffixes}, $(patsubst %, ${SourceDir}/%.$s, ${SharedModules})))))
endif

ifneq (${IgnoreModules},)
  SharedModules := $(filter-out ${IgnoreModules},${SharedModules})
endif

ifneq ($(wildcard ${SourceDir}/perl),)
  SharedModules += $(filter-out perl/wrap-%, $(basename $(call _list_CXX_sources,${SourceDir},perl)))
  SourceSubdirs := perl
endif

SharedObjects := $(addsuffix $O, ${SharedModules})

CXXflags += -DPOLYMAKE_APPNAME=${AppName}

ifdef ExtensionTop
  IncludesXX += $(foreach ext, ${ExtensionTop} ${ImportExts}, $(addprefix ${ext}/, include/app-wrappers include/apps))
endif
ifneq (${AppTop},${ProjectTop})
  IncludesXX += $(addprefix ${AppTop}/, include/app-wrappers include/apps)
else ifdef InSourceTree
  IncludesXX += $(addprefix ${ProjectTop}/, include/app-wrappers include/apps)
endif

Libs := -lmpfr -lgmp ${Libs}

OwnShared := ${BuildDir}/lib/${AppName}$S

ifdef ExtensionTop
  # recompile everything if the extension was reconfigured
  ${SharedObjects} : ${BuildDir}/conf.make
endif

WithWrappers.cc  := $(patsubst wrap-%,%$O,$(basename $(notdir $(wildcard ${SourceDir}/perl/wrap-*.cc))))
WithWrappers.C   := $(patsubst wrap-%,%$O,$(basename $(notdir $(wildcard ${SourceDir}/perl/wrap-*.C))))
WithWrappers.cpp := $(patsubst wrap-%,%$O,$(basename $(notdir $(wildcard ${SourceDir}/perl/wrap-*.cpp))))
${WithWrappers.cc}  : %$O : ${SourceDir}/perl/wrap-%.cc
${WithWrappers.C}   : %$O : ${SourceDir}/perl/wrap-%.C
${WithWrappers.cpp} : %$O : ${SourceDir}/perl/wrap-%.cpp
${WithWrappers.cc} ${WithWrappers.C} ${WithWrappers.cpp} : withWrapper = ${SourceDir}/perl/wrap-$(notdir $<)

# ignore deleted wrappers in dependences
${SourceDir}/perl/wrap-%.cc ${SourceDir}/perl/wrap-%.C ${SourceDir}/perl/wrap-%.cpp :
	@echo wrapper file $@ mentioned in .dependences does not exist - ignored

ifdef ExternalLibs
  .PHONY: compile-external

  define _MakeExternalLib
      $(MAKE) --print-directory -C ${BuildDir}/external/$(1) Debug=${Debug} $(2)

  endef
  define _MakeExternal
      $(foreach x,${ExternalLibs},$(call _MakeExternalLib,$x,$(1)))
  endef

  # build process of an external library might produce header files needed for our clients
  ${SharedObjects} : | compile-external

  compile-external:
	+@$(call _MakeExternal,compile)

  clean::
	+@$(call _MakeExternal,clean)
endif

${OwnShared} : $(if ${OnlyModules}, $(filter $(addsuffix $O,${OnlyModules}), ${SharedObjects}), ${SharedObjects})
	${CXX} ${LDsharedFlags} -o $@ ${SharedObjects} ${LDFLAGS} ${LIBS}

compile : ${OwnShared}

install-src:
	${PERL} ${INSTALL_PL} -m ${DirMask} -U -W ${SourceTop}/include/app-wrappers/polymake/${AppName} ${SourceTop}/apps/${AppName}/include ${InstallInc}/polymake/${AppName}
	${PERL} ${INSTALL_PL} -m ${DirMask} -U -X perl ${SourceTop}/apps/${AppName}/src ${InstallTop}${InstallSuffix}/apps/${AppName}/src

install : compile
	${PERL} ${INSTALL_PL} -m 555 ${OwnShared} ${InstallArch}${InstallSuffix}/lib

clean::
	rm -f ${OwnShared}
